#include <windows.h>
#include <stdio.h>
#include <atlbase.h>
extern CComModule _Module;
#include <atlcom.h>
using namespace ATL;

// Microsoft Office Core
#import "libs\MSO9.DLL" rename_namespace("Office") rename("RGB", "MSORGB"), named_guids
using namespace Office;

// Outlook
#import "libs\MSOUTL9.OLB" rename_namespace("Outlook"), named_guids
using namespace Outlook;

// SendPersonLib
#import "libs\SendPerson.dll" rename_namespace("CSP"), named_guids
using namespace CSP;


// Initialisation of the structure describing the handler of events of ITMMailItem
_ATL_FUNC_INFO OnEvaluateMacrosInfo = {CC_STDCALL, VT_EMPTY, 2, {VT_DISPATCH, VT_DISPATCH}};

class CMacrosEvaluator;
typedef IDispEventSimpleImpl<1, CMacrosEvaluator, &__uuidof(CSP::_ITMMailItemEvents)> OnEvaluateMacros;

///////////////////////////////////////////////////////////////////////////////
// Class CMacrosEvaluator is intended to calculate the custom values of macro
// in our example it is a JOB_TITLE
class CMacrosEvaluator : public OnEvaluateMacros
{
public:
	// Constructor
	CMacrosEvaluator(Outlook::_Application* lpApplication, CSP::ITMMailItem* lpTMMailItem)
	{
		HRESULT hr = S_OK;
		try
		{
			m_bEventAdvised = FALSE;
			m_spApplication = lpApplication;
			m_spTMMailItem  = lpTMMailItem;

			// "Advise" for event EvaluateMacros
			hr = DispEventAdvise(m_spTMMailItem);
			if(SUCCEEDED(hr)) m_bEventAdvised = TRUE;
		}
		catch(...){ printf("Error!;\n"); }
	};

	~CMacrosEvaluator(void)
	{
		ForceUnadvise();
	};

	void ForceUnadvise()
	{
		// "Unadvise" for event EvaluateMacros
		if(m_bEventAdvised)
			DispEventUnadvise(m_spTMMailItem);

		m_spApplication = NULL;
		m_spTMMailItem = NULL;
		m_bEventAdvised = FALSE;
	}

	BEGIN_SINK_MAP(CMacrosEvaluator)
		SINK_ENTRY_INFO(1, __uuidof(CSP::_ITMMailItemEvents), 0x1, OnEvaluateMacros, &OnEvaluateMacrosInfo)
	END_SINK_MAP()

	// Handler for EvaluateMacros
	void __stdcall OnEvaluateMacros(LPDISPATCH lpDispPersonData, LPDISPATCH lpDispMacros)
	{
		ITMPersonDataPtr	spPersonData(lpDispPersonData);
		ITMMacrosPtr		spMacros(lpDispMacros);

		try
		{
			// Checking for calculated macro
			if(spMacros->Name == _bstr_t("JOB_TITLE"))
			{
				// Look for item "Contact" in the contact folder
				MAPIFolderPtr spContactsFolder = m_spApplication->Session->GetDefaultFolder(olFolderContacts);
				_ItemsPtr spContactItems = spContactsFolder->Items;
				
				char szFilter[256] = {0};
				sprintf(szFilter, "[Email1Address]=%s", (LPCTSTR)(spPersonData->Address));
				_ContactItemPtr spRecipientContact = spContactItems->Find(_bstr_t(szFilter));

				// Update "Value" by new data
				spMacros->Value = spRecipientContact->JobTitle;
			}
		}
		catch(...)
		{
			spMacros->Value = _bstr_t("[Macros evaluation error]");
		}
	}

	_ApplicationPtr m_spApplication;
	ITMMailItemPtr  m_spTMMailItem;
	BOOL			m_bEventAdvised;
};



///////////////////////////////////////////////////////////////////////////////
// Main function
void main()
{
	HRESULT hr = S_OK;

	BOOL bComInitialized = FALSE;

	try
	{
		// COM initialization
		hr = CoInitialize(NULL);
		if(FAILED(hr)) throw hr;
		bComInitialized = TRUE;

		// Create object Outlook.Application
		_ApplicationPtr spApplication("Outlook.Application", NULL, CLSCTX_LOCAL_SERVER);

		// Begin MAPI session (necessary condition)
		_NameSpacePtr spNameSpace = spApplication->GetNamespace(_bstr_t("MAPI"));
		spNameSpace->Logon();

		// Create new message of Microsoft Outlook
		_MailItemPtr spMailItem = spApplication->CreateItem(olMailItem);

		// Put recipient address into the "To:" field of the message
		spMailItem->To = _bstr_t("Some Recipient <somerecipient@company.com>; Another Recipient <another@company.com>");

		// Put the data containing the macros into the "Subject" and "Body" fields
		spMailItem->Subject = _bstr_t("Test message for %NAME% (%EMAIL%)");
		spMailItem->Body = _bstr_t("Hello %NAME% (%EMAIL%)!\r\nI'm glad mail to %JOB_TITLE%");

		// Create object ITMMailItem of Send Personally Component
		ITMMailItemPtr spCSPItem("SendPerson.TMMailItem");
 
		// Put the reference to the message of Microsoft Outlook into the properties "Item"
		spCSPItem->Item = spMailItem;

		// Create the macros CSP used in the message
		spCSPItem->AddMacros(_bstr_t("NAME"), _variant_t((long)dfRecipientName), VARIANT_TRUE);
		spCSPItem->AddMacros(_bstr_t("EMAIL"), _variant_t((long)dfRecipientEmail), VARIANT_TRUE);
		spCSPItem->AddMacros(_bstr_t("FIRSTNAME"), _variant_t((long)dfRecipientFirstName), VARIANT_TRUE);
		spCSPItem->AddMacros(_bstr_t("LASTNAME"), _variant_t((long)dfRecipientLastName), VARIANT_TRUE);

		spCSPItem->AddMacros(_bstr_t("JOB_TITLE"), _variant_t((long)dfOwnerEvaluate), VARIANT_TRUE);

		CMacrosEvaluator MacrosEvaluator(spApplication, spCSPItem);

		spCSPItem->put_Lic(_bstr_t("Here you license key"));

		// Send message
		spCSPItem->SendPersonally(NULL, VARIANT_TRUE, _bstr_t(""), _bstr_t(""));

		MacrosEvaluator.ForceUnadvise();

		// Delete the source message
		spMailItem->Delete();

		printf("Successfully!\n");
	}
	catch(_com_error& e)
	{
		printf("Error = %s;\n", e.ErrorMessage());
	}
	catch(HRESULT hResult)
	{
		printf("Error 0x%8X\n", hResult);
	}

	if(bComInitialized) CoUninitialize();
}
